@@ -85,8 +85,8 @@ public:
     QPointF startPos;
     QPointF initialTopLeft;
     MouseLocation dragLocation = MouseLocation::None;
-    QVector<ScreenImage> screenImages;
-    QPixmap pixmap;
+    QImage image;
+    QVector<CanvasImage> screenImages;
     qreal devicePixelRatio = 1;
     qreal devicePixelRatioI = 1;
     QPointF mousePos;
@@ -439,62 +439,41 @@ qreal SelectionEditor::dprRound(qreal value, qreal dpr) const
     return std::round(value * dpr) / dpr;
 }
 
-QImage SelectionEditor::imageForScreen(QScreen *screen)
-{
-    for (const auto &si : qAsConst(d->screenImages)) {
-        if (si.screen == screen) {
-            return si.image;
-        }
-    }
-    return QImage();
-}
-
-QImage SelectionEditor::imageForScreenName(const QString &name)
-{
-    for (const auto &si : qAsConst(d->screenImages)) {
-        if (si.screen->name() == name) {
-            return si.image;
-        }
-    }
-    return QImage();
-}
-
 qreal SelectionEditor::dprRound(qreal value) const
 {
     return dprRound(value, d->devicePixelRatio);
 }
 
-void SelectionEditor::setScreenImages(const QVector<ScreenImage> &screenImages)
+void SelectionEditor::setScreenImages(const QVector<CanvasImage> &screenImages)
 {
     QVector<QPoint> translatedPoints;
     QRect screensRect;
     for (int i = 0; i < screenImages.length(); ++i) {
-        const QScreen *screen = screenImages[i].screen;
+        const QPoint &screenPos = screenImages[i].rect.topLeft().toPoint();
         const QImage &image = screenImages[i].image;
-        const qreal dpr = screenImages[i].devicePixelRatio;
+        const qreal dpr = image.devicePixelRatio();
 
         QRect virtualScreenRect;
         if (KWindowSystem::isPlatformX11()) {
-            virtualScreenRect = QRect(screen->geometry().topLeft(), image.size());
+            virtualScreenRect = QRect(screenPos, image.size());
         } else {
             // `QSize / qreal` divides the int width and int height by the qreal factor,
             // then rounds the results to the nearest int.
-            virtualScreenRect = QRect(screen->geometry().topLeft(), image.size() / dpr);
+            virtualScreenRect = QRect(screenPos, image.size() / dpr);
         }
         screensRect = screensRect.united(virtualScreenRect);
 
-        translatedPoints.append(screen->geometry().topLeft());
+        translatedPoints.append(screenPos);
     }
 
     d->screenImages = screenImages;
 
     // compute coordinates after scaling
     for (int i = 0; i < screenImages.length(); ++i) {
-        const QScreen *screen = screenImages[i].screen;
         const QImage &image = screenImages[i].image;
-        const QPoint &p = screen->geometry().topLeft();
+        const QPoint &p = screenImages[i].rect.topLeft().toPoint();
         const QSize &size = image.size();
-        const double dpr = screenImages[i].devicePixelRatio;
+        const double dpr = image.devicePixelRatio();
         if (!qFuzzyCompare(dpr, 1.0)) {
             // must update all coordinates of next rects
             int newWidth = size.width();
@@ -505,7 +484,7 @@ void SelectionEditor::setScreenImages(const QVector<ScreenImage> &screenImages)
 
             // for the next size
             for (int i2 = i; i2 < screenImages.length(); ++i2) {
-                auto point = screenImages[i2].screen->geometry().topLeft();
+                auto point = screenImages[i2].rect.topLeft();
 
                 if (point.x() >= newWidth + p.x() - deltaX) {
                     translatedPoints[i2].setX(translatedPoints[i2].x() + deltaX);
@@ -517,14 +496,17 @@ void SelectionEditor::setScreenImages(const QVector<ScreenImage> &screenImages)
         }
     }
 
-    d->pixmap = QPixmap(screensRect.size());
-    QPainter painter(&d->pixmap);
+    d->image = QImage(screensRect.size(), QImage::Format_ARGB32);
+    d->image.fill(Qt::black);
+    QPainter painter(&d->image);
     // Don't enable SmoothPixmapTransform, we want crisp graphics.
     for (int i = 0; i < screenImages.length(); ++i) {
         // Geometry can have negative coordinates,
         // so it is necessary to subtract the upper left point,
         // because coordinates on the widget are counted from 0.
-        painter.drawImage(translatedPoints[i] - screensRect.topLeft(), screenImages[i].image);
+        QImage image = screenImages[i].image;
+        image.setDevicePixelRatio(1);
+        painter.drawImage(translatedPoints[i] - screensRect.topLeft(), image);
     }
 
     if (d->screensRect != screensRect) {
@@ -535,12 +517,12 @@ void SelectionEditor::setScreenImages(const QVector<ScreenImage> &screenImages)
     Q_EMIT screenImagesChanged();
 }
 
-QVector<ScreenImage> SelectionEditor::screenImages() const
+QVector<CanvasImage> SelectionEditor::screenImages() const
 {
     return d->screenImages;
 }
 
-bool SelectionEditor::acceptSelection()
+bool SelectionEditor::acceptSelection(ExportManager::Actions actions)
 {
     if (d->screenImages.isEmpty()) {
         return false;
@@ -557,11 +539,11 @@ bool SelectionEditor::acceptSelection()
     spectacleCore->annotationDocument()->cropCanvas(d->selection->alignedRect(1));
 
     if (KWindowSystem::isPlatformX11()) {
-        d->pixmap.setDevicePixelRatio(qGuiApp->devicePixelRatio());
-        if (scaledCropRegion.size() != d->pixmap.size()) {
-            Q_EMIT spectacleCore->grabDone(d->pixmap.copy(scaledCropRegion));
+        d->image.setDevicePixelRatio(qGuiApp->devicePixelRatio());
+        if (scaledCropRegion.size() != d->image.size()) {
+            Q_EMIT spectacleCore->grabDone(d->image.copy(scaledCropRegion), actions);
         } else {
-            Q_EMIT spectacleCore->grabDone(d->pixmap);
+            Q_EMIT spectacleCore->grabDone(d->image, actions);
         }
     } else { // Wayland case
         // QGuiApplication::devicePixelRatio() is calculated by getting the highest screen DPI
@@ -569,17 +551,17 @@ bool SelectionEditor::acceptSelection()
 
         QRect selectionRect = d->selection->alignedRect();
         QSize selectionSize = selectionRect.size();
-        QPixmap output(selectionSize * maxDpr);
+        QImage output(selectionSize * maxDpr, QImage::Format_ARGB32);
+        output.fill(Qt::black);
         QPainter painter(&output);
         // Don't enable SmoothPixmapTransform, we want crisp graphics
 
         for (auto it = d->screenImages.constBegin(); it != d->screenImages.constEnd(); ++it) {
-            const auto screen = it->screen;
-            const auto &screenRect = screen->geometry();
+            const QRect &screenRect = it->rect.toRect();
 
             if (selectionRect.intersects(screenRect)) {
                 const QPoint pos = screenRect.topLeft();
-                const qreal dpr = it->devicePixelRatio;
+                const qreal dpr = it->image.devicePixelRatio();
 
                 QRect intersected = screenRect.intersected(selectionRect);
 
@@ -589,7 +571,7 @@ bool SelectionEditor::acceptSelection()
                 pixelOnScreenIntersected.setWidth(intersected.width() * dpr);
                 pixelOnScreenIntersected.setHeight(intersected.height() * dpr);
 
-                QPixmap screenOutput = QPixmap::fromImage(it->image.copy(pixelOnScreenIntersected));
+                QImage screenOutput = it->image.copy(pixelOnScreenIntersected);
 
                 // FIXME: this doesn't seem correct
                 if (intersected.size() == selectionSize) {
@@ -597,23 +579,19 @@ bool SelectionEditor::acceptSelection()
                     // keep native screen resolution
                     // we need to set the pixmap dpr to be able to properly align with annotations
                     screenOutput.setDevicePixelRatio(dpr);
-                    Q_EMIT spectacleCore->grabDone(screenOutput);
+                    Q_EMIT spectacleCore->grabDone(screenOutput, actions);
                     return true;
                 }
 
                 // upscale the image according to max screen dpr, to keep the image not distorted
                 output.setDevicePixelRatio(maxDpr);
-                const auto dprI = maxDpr / dpr;
-                QBrush brush(screenOutput);
-                brush.setTransform(QTransform::fromScale(dprI, dprI));
                 intersected.moveTopLeft((intersected.topLeft() - selectionRect.topLeft()) * maxDpr);
                 intersected.setSize(intersected.size() * maxDpr);
-                painter.setBrushOrigin(intersected.topLeft());
-                painter.fillRect(intersected, brush);
+                painter.drawImage(intersected, screenOutput);
             }
         }
 
-        Q_EMIT spectacleCore->grabDone(output);
+        Q_EMIT spectacleCore->grabDone(output, actions);
     }
 
     return true;
@@ -710,7 +688,7 @@ void SelectionEditor::hoverMoveEvent(QQuickItem *item, QHoverEvent *event)
     if (!item->window() || !item->window()->screen()) {
         return;
     }
-    d->mousePos = event->posF() + item->window()->screen()->geometry().topLeft();
+    d->mousePos = event->posF() + item->window()->screen()->geometry().topLeft() / (KWindowSystem::isPlatformWayland() ? 1 : devicePixelRatio());
     Q_EMIT mousePositionChanged();
     d->setMouseCursor(item, d->mousePos);
 }
@@ -733,7 +711,7 @@ void SelectionEditor::mousePressEvent(QQuickItem *item, QMouseEvent *event)
         }
         item->setFocus(true);
         const bool wasMagnifierAllowed = d->magnifierAllowed;
-        d->mousePos = event->localPos() + item->window()->screen()->geometry().topLeft();
+        d->mousePos = event->localPos() + item->window()->screen()->geometry().topLeft() / (KWindowSystem::isPlatformWayland() ? 1 : devicePixelRatio());
         Q_EMIT mousePositionChanged();
         auto newDragLocation = d->mouseLocation(d->mousePos);
         if (d->dragLocation != newDragLocation) {
@@ -786,7 +764,7 @@ void SelectionEditor::mouseMoveEvent(QQuickItem *item, QMouseEvent *event)
         return;
     }
 
-    d->mousePos = event->localPos() + item->window()->screen()->geometry().topLeft();
+    d->mousePos = event->localPos() + item->window()->screen()->geometry().topLeft() / (KWindowSystem::isPlatformWayland() ? 1 : devicePixelRatio());
     Q_EMIT mousePositionChanged();
     const bool wasMagnifierAllowed = d->magnifierAllowed;
     d->magnifierAllowed = true;
